storage_capacity = 9;
num_possible_states = storage_capacity + 1;
b = 0.2; % birth probability / probability of new request
d = 0.5; % death probability / probability of single request completion

% CREATION OF TRANSITION PROBABILITY MATRIX
T = zeros(num_possible_states, num_possible_states);
for i=1:num_possible_states
    curr_state = i-1;
    for j=1:num_possible_states
        next_state = j-1;
        if next_state == curr_state + 1
            T(i,j) = b;
        elseif next_state == curr_state - 1
            T(i,j) = d;
        elseif next_state == curr_state 
            if curr_state == 0
                T(i,j) = 1-b;
            elseif curr_state == storage_capacity
                T(i,j) = 1-d;
            else
                T(i,j) = 1-b-d;
            end
        else
            T(i,j) = 0;
        end
    end
end

% Part a)
pi_0 = zeros(1,num_possible_states);
pi_0(:) = 1/num_possible_states; % Uniform distribution over all states
pi_1 = pi_0 * T;

% Part b)
time_steps = 100;
pi_0 = zeros(1,num_possible_states);
pi_0(:) = 1/num_possible_states;
prob_vectors = zeros(time_steps,num_possible_states); % Initialize table storing probability vectors at each time step
absolute_changes = zeros(1,time_steps); % Initialize vector storing absolute changes at each time step
for i=1:100
    if i==1
        pi_prev = pi_0;
        pi_i = pi_0 * T;
    else
        pi_prev = prob_vectors(i-1,:);
        pi_i = pi_prev * T;
    end
    prob_vectors(i,:) = pi_i;
    e = sum(abs(pi_i - pi_prev));
    absolute_changes(i) = e;
end
plot(absolute_changes)

% Part c)
time_steps = 100;
pi_0 = zeros(1,num_possible_states);
pi_0(1) = 1; % Queue is empty at time 0 with probability 1.
prob_vectors = zeros(time_steps,num_possible_states); % Initialize table storing probability vectors at each time step
absolute_changes = zeros(1,time_steps); % Initialize vector storing absolute changes at each time step
for i=1:100
    if i==1
        pi_prev = pi_0;
        pi_i = pi_0 * T;
    else
        pi_prev = prob_vectors(i-1,:);
        pi_i = pi_prev * T;
    end
    prob_vectors(i,:) = pi_i;
    e = sum(abs(pi_i - pi_prev));
    absolute_changes(i) = e;
end
plot(absolute_changes)

% Part d)
num_experiments = 1000;
time_steps = 100;
curr_state = 0; % x_0 = 0
frequencies = zeros(time_steps, num_possible_states); % Initialize table storing experiment results
for i=1:num_experiments
    for j=1:time_steps
        possible_next_states = 0:9;
        row_num = curr_state + 1; 
        next_state_probabilities = T(row_num,:); % Transition probabilities corresponding to current state
        next_state = randsample(possible_next_states,1,true,next_state_probabilities); % Sample next state according to transition probabilities given current state
        col_num = next_state + 1;
        frequencies(j,col_num) = frequencies(j,col_num) + 1; % Update table
        curr_state = next_state;
    end
end
delta = zeros(1,num_possible_states); % Initialize vector for storing value of function evalulated at each state
true_probs = prob_vectors;
n = 100; % The time step we're specifically interested in, i.e. the last time step
for i=1:num_possible_states
    curr_state_freq = frequencies(n,i);
    curr_state_estimated_prob = curr_state_freq/num_experiments;
    curr_state_true_prob = true_probs(n,i);
    delta(i) = abs(curr_state_estimated_prob - curr_state_true_prob)/curr_state_true_prob; % Evaluate function
end
plot(delta)

% Part e) - Repeat part b) after changing d to be 0.1 at the top of the
% file